i.icon.vede:before {
  content: "Ⰲ";
}

.vest main.container {
  position: relative;
  margin: auto;
  /* top right bottom left */
  padding: 4em 1.5em 0em 1.5em;
  background: #FAFAFA;
  height: 100%;
 
}

#talks.sidebar {
    background: #FAFAFA;
}
#talks.menu.header.item {
  width:100%;
}

#talk_topic {
 /* height:2em; */
}
#messages {
  /* top right bottom left */
  padding: 1em;
  height: 55vh;
  overflow: auto;
}
#messages .ul.list {}

form#message_form.ui.form.segment .field textarea {
  min-height:2em;
  height: 5em;
  padding: 1ex;
}
